import PropTypes from 'prop-types';
import React, { Component, useState } from 'react';

function stepBelow(x, step) {
  return x - x % step;
}
function findNearestIndex(arr, x) {
  var low = 0;
  var high = arr.length - 1;

  if (arr.length <= 1) {
    return high;
  }

  if (arr[low] > arr[high]) {
    throw "Data must be sorted in ascending order";
  } // binary search


  while (low < high) {
    var mid = (low + high) / 2 >> 0;

    if (arr[mid] > x) {
      high = mid;
    } else {
      low = mid + 1;
    }
  }

  if (arr[low - 1] !== undefined) {
    var midX = (arr[low] + arr[low - 1]) / 2;

    if (x < midX) {
      return low - 1;
    } else {
      return low;
    }
  }

  return low;
}
function isInside(rect, _ref) {
  var x = _ref.x,
      y = _ref.y;

  if (!x || x < rect.left || x > rect.left + rect.width) {
    return false;
  }

  if (!y || y < rect.top || y > rect.top + rect.height) {
    return false;
  }

  return true;
}
function linear(dy, dx, x, y0) {
  return dy / dx * x + y0;
}
function getCanvasX(width, left, xMax, xMin, x) {
  var dx = xMax - xMin;
  return linear(width, dx, x, left - width * xMin / dx);
}
function getCanvasY(height, top, yMax, yMin, y) {
  var dy = yMax - yMin;
  return linear(-height, dy, y, top + height * yMax / dy);
}
function getX(width, left, xMax, xMin, canvasX) {
  var dx = xMax - xMin;
  return linear(dx, width, canvasX, xMin - dx / width * left);
}
function getY(height, top, yMax, yMin, canvasY) {
  var dy = yMax - yMin;
  return linear(-dy, height, canvasY, yMax + dy / height * top);
}

var math = /*#__PURE__*/Object.freeze({
  stepBelow: stepBelow,
  findNearestIndex: findNearestIndex,
  isInside: isInside,
  linear: linear,
  getCanvasX: getCanvasX,
  getCanvasY: getCanvasY,
  getX: getX,
  getY: getY
});

var TICK_TEXT_PADDING = 10;

function drawTick(ctx, layout, props, x) {
  var _layout$xAxis = layout.xAxis,
      top = _layout$xAxis.top,
      left = _layout$xAxis.left,
      width = _layout$xAxis.width,
      height = _layout$xAxis.height;
  var xAxisAt = props.xAxisAt,
      xAxisFont = props.xAxisFont,
      xAxisLineColor = props.xAxisLineColor,
      xAxisTextColor = props.xAxisTextColor,
      xTickLength = props.xTickLength,
      renderXTick = props.renderXTick,
      xMin = props.xMin,
      xMax = props.xMax;
  var canvasX = getCanvasX(width, left, xMax, xMin, x);
  ctx.font = xAxisFont;
  ctx.fillStyle = xAxisTextColor;
  ctx.strokeStyle = xAxisLineColor;
  ctx.textAlign = "center";
  ctx.textBaseline = "middle";

  if (xAxisAt == "top") {
    ctx.beginPath();
    ctx.moveTo(canvasX, top + height);
    ctx.lineTo(canvasX, top + height - xTickLength);
    ctx.stroke();
    ctx.fillText(renderXTick(x), canvasX, top + height - xTickLength - TICK_TEXT_PADDING);
  } else if (xAxisAt == "bottom") {
    ctx.beginPath();
    ctx.moveTo(canvasX, top);
    ctx.lineTo(canvasX, top + xTickLength);
    ctx.stroke();
    ctx.fillText(renderXTick(x), canvasX, top + xTickLength + TICK_TEXT_PADDING);
  }
}

function drawLine(ctx, layout, props, x) {
  var _layout$graph = layout.graph,
      left = _layout$graph.left,
      top = _layout$graph.top,
      width = _layout$graph.width,
      height = _layout$graph.height;
  var xLineColor = props.xLineColor,
      xMax = props.xMax,
      xMin = props.xMin;
  var canvasX = getCanvasX(width, left, xMax, xMin, x);
  ctx.strokeStyle = xLineColor;
  ctx.beginPath();
  ctx.moveTo(canvasX, top);
  ctx.lineTo(canvasX, top + height);
  ctx.stroke();
}

function draw(ctx, layout, props) {
  var _layout$xAxis2 = layout.xAxis,
      top = _layout$xAxis2.top,
      left = _layout$xAxis2.left,
      width = _layout$xAxis2.width,
      height = _layout$xAxis2.height;
  var xAxisAt = props.xAxisAt,
      xAxisLineColor = props.xAxisLineColor,
      xTicks = props.xTicks,
      xTickInterval = props.xTickInterval,
      showXLine = props.showXLine,
      xMin = props.xMin,
      xMax = props.xMax; // style x axis line

  ctx.lineWidth = 1;
  ctx.strokeStyle = xAxisLineColor;

  if (xAxisAt == "top") {
    ctx.beginPath();
    ctx.moveTo(left, top + height);
    ctx.lineTo(left + width, top + height);
    ctx.stroke();
  } else if (xAxisAt == "bottom") {
    ctx.beginPath();
    ctx.moveTo(left, top);
    ctx.lineTo(left + width, top);
    ctx.stroke();
  }

  if (xTickInterval > 0) {
    var x0 = stepBelow(xMin, xTickInterval);

    for (var x = x0; x <= xMax; x += xTickInterval) {
      if (x < xMin) {
        continue;
      }

      drawTick(ctx, layout, props, x);

      if (showXLine) {
        drawLine(ctx, layout, props, x);
      }
    }
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = xTicks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _x = _step.value;

      if (_x < xMin || _x > xMax) {
        continue;
      }

      drawTick(ctx, layout, props, _x);

      if (showXLine) {
        drawLine(ctx, layout, props, _x);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

var xAxis = /*#__PURE__*/Object.freeze({
  draw: draw
});

var TICK_TEXT_PADDING$1 = 5;

function drawTick$1(ctx, layout, props, y) {
  var _layout$yAxis = layout.yAxis,
      top = _layout$yAxis.top,
      left = _layout$yAxis.left,
      height = _layout$yAxis.height,
      width = _layout$yAxis.width;
  var yAxisAt = props.yAxisAt,
      yAxisLineColor = props.yAxisLineColor,
      yAxisFont = props.yAxisFont,
      yAxisTextColor = props.yAxisTextColor,
      yMax = props.yMax,
      yMin = props.yMin,
      yTickLength = props.yTickLength,
      renderYTick = props.renderYTick;
  var canvasY = getCanvasY(height, top, yMax, yMin, y);
  ctx.strokeStyle = yAxisLineColor;
  ctx.font = yAxisFont;
  ctx.fillStyle = yAxisTextColor;
  ctx.textAlign = yAxisAt == "left" ? "right" : "left";
  ctx.textBaseline = "middle";

  if (yAxisAt == "left") {
    ctx.beginPath();
    ctx.moveTo(left + width, canvasY);
    ctx.lineTo(left + width - yTickLength, canvasY);
    ctx.stroke();
    ctx.fillText(renderYTick(y), left + width - yTickLength - TICK_TEXT_PADDING$1, canvasY);
  } else if (yAxisAt == "right") {
    ctx.beginPath();
    ctx.moveTo(left, canvasY);
    ctx.lineTo(left + yTickLength, canvasY);
    ctx.stroke();
    ctx.fillText(renderYTick(y), left + yTickLength + TICK_TEXT_PADDING$1, canvasY);
  }
}

function drawLine$1(ctx, layout, props, y) {
  var _layout$graph = layout.graph,
      top = _layout$graph.top,
      left = _layout$graph.left,
      height = _layout$graph.height,
      width = _layout$graph.width;
  var yLineColor = props.yLineColor,
      yMax = props.yMax,
      yMin = props.yMin;
  var canvasY = getCanvasY(height, top, yMax, yMin, y);
  ctx.strokeStyle = yLineColor;
  ctx.beginPath();
  ctx.moveTo(left, canvasY);
  ctx.lineTo(left + width, canvasY);
  ctx.stroke();
}

function draw$1(ctx, layout, props) {
  var _layout$yAxis2 = layout.yAxis,
      top = _layout$yAxis2.top,
      left = _layout$yAxis2.left,
      height = _layout$yAxis2.height,
      width = _layout$yAxis2.width;
  var showYLine = props.showYLine,
      yAxisAt = props.yAxisAt,
      yAxisLineColor = props.yAxisLineColor,
      yTicks = props.yTicks,
      yTickInterval = props.yTickInterval,
      yAxisFont = props.yAxisFont,
      yAxisTextColor = props.yAxisTextColor,
      yMin = props.yMin,
      yMax = props.yMax; // style y axis line

  ctx.lineWidth = 1;
  ctx.strokeStyle = yAxisLineColor;

  if (yAxisAt == "left") {
    ctx.beginPath();
    ctx.moveTo(left + width, top);
    ctx.lineTo(left + width, top + height);
    ctx.stroke();
  } else if (yAxisAt == "right") {
    ctx.beginPath();
    ctx.moveTo(left, top);
    ctx.lineTo(left, top + height);
    ctx.stroke();
  }

  if (yTickInterval > 0) {
    var y0 = stepBelow(yMin, yTickInterval);

    for (var y = y0; y <= yMax; y += yTickInterval) {
      if (y < yMin) {
        continue;
      }

      drawTick$1(ctx, layout, props, y);

      if (showYLine) {
        drawLine$1(ctx, layout, props, y);
      }
    }
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = yTicks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _y = _step.value;

      if (_y < yMin || _y > yMax) {
        continue;
      }

      drawTick$1(ctx, layout, props, _y);

      if (showYLine) {
        drawLine$1(ctx, layout, props, _y);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

var yAxis = /*#__PURE__*/Object.freeze({
  draw: draw$1
});

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var propTypes = {
  x: PropTypes.number,
  width: PropTypes.number.isRequired,
  height: PropTypes.number.isRequired,
  backgroundColor: PropTypes.string.isRequired,
  color: PropTypes.string.isRequired,
  font: PropTypes.string.isRequired,
  textPadding: PropTypes.number.isRequired,
  render: PropTypes.func.isRequired,
  drawLine: PropTypes.bool.isRequired,
  lineWidth: PropTypes.number.isRequired,
  lineColor: PropTypes.string.isRequired
};
var defaultProps = {
  width: 50,
  height: 20,
  backgroundColor: "white",
  font: "",
  color: "black",
  textPadding: 10,
  render: function render() {
    return "";
  },
  drawLine: true,
  lineWidth: 1,
  lineColor: "black"
};

function setDefaults(props) {
  return _objectSpread2({}, defaultProps, {}, props);
}

function getTop(label, layout, props) {
  var graph = layout.graph;
  var xAxisAt = props.xAxisAt,
      xTickLength = props.xTickLength;

  if (xAxisAt == "top") {
    return graph.top - label.height - xTickLength;
  }

  if (xAxisAt == "bottom") {
    return graph.top + graph.height + xTickLength;
  }

  return 0;
}

function getLineStart(label, layout, props) {
  var graph = layout.graph;
  var xAxisAt = props.xAxisAt,
      xTickLength = props.xTickLength;

  if (xAxisAt == "top") {
    return graph.top - xTickLength;
  }

  if (xAxisAt == "bottom") {
    return graph.top + graph.height + xTickLength;
  }

  return 0;
}

function getLineEnd(label, layout, props) {
  var graph = layout.graph;
  var xAxisAt = props.xAxisAt;

  if (xAxisAt == "top") {
    return graph.top + graph.height;
  }

  if (xAxisAt == "bottom") {
    return graph.top;
  }

  return 0;
}

function draw$2(ctx, layout, label, props) {
  label = setDefaults(label);
  PropTypes.checkPropTypes(propTypes, label, "prop", "x-label");
  var _label = label,
      x = _label.x,
      width = _label.width,
      height = _label.height,
      backgroundColor = _label.backgroundColor,
      font = _label.font,
      color = _label.color,
      textPadding = _label.textPadding,
      render = _label.render,
      drawLine = _label.drawLine,
      lineWidth = _label.lineWidth,
      lineColor = _label.lineColor;
  var graph = layout.graph;
  var xMin = props.xMin,
      xMax = props.xMax;

  if (x == undefined) {
    return;
  }

  var canvasX = getCanvasX(graph.width, graph.left, xMax, xMin, x);
  var left = canvasX - Math.round(width / 2);
  var top = getTop(label, layout, props); // label box

  ctx.fillStyle = backgroundColor;
  ctx.fillRect(left, top, width, height); // text

  ctx.font = font;
  ctx.fillStyle = color;
  ctx.textAlign = "center";
  ctx.textBaseline = "middle";
  ctx.fillText(render(x), left + width / 2, top + textPadding);

  if (drawLine) {
    ctx.lineWidth = lineWidth;
    ctx.strokeStyle = lineColor;
    var lineStart = getLineStart(label, layout, props);
    var lineEnd = getLineEnd(label, layout, props);
    ctx.beginPath();
    ctx.moveTo(left + width / 2, lineStart);
    ctx.lineTo(left + width / 2, lineEnd);
    ctx.stroke();
  }
}

var xLabel = /*#__PURE__*/Object.freeze({
  draw: draw$2
});

var propTypes$1 = {
  y: PropTypes.number,
  width: PropTypes.number.isRequired,
  height: PropTypes.number.isRequired,
  backgroundColor: PropTypes.string.isRequired,
  color: PropTypes.string.isRequired,
  font: PropTypes.string.isRequired,
  textPadding: PropTypes.number.isRequired,
  render: PropTypes.func.isRequired,
  drawLine: PropTypes.bool.isRequired,
  lineWidth: PropTypes.number.isRequired,
  lineColor: PropTypes.string.isRequired
};
var defaultProps$1 = {
  width: 50,
  height: 20,
  backgroundColor: "white",
  font: "",
  color: "black",
  textPadding: 10,
  render: function render() {
    return "";
  },
  drawLine: true,
  lineColor: "black",
  lineWidth: 1
};

function setDefaults$1(props) {
  return _objectSpread2({}, defaultProps$1, {}, props);
}

function getLeft(label, layout, props) {
  var graph = layout.graph;
  var yAxisAt = props.yAxisAt,
      yTickLength = props.yTickLength;

  if (yAxisAt == "left") {
    return graph.left - label.width - yTickLength;
  }

  if (yAxisAt == "right") {
    return graph.left + graph.width + yTickLength;
  }

  return 0;
}

function getTextAlign(props) {
  var yAxisAt = props.yAxisAt;

  switch (yAxisAt) {
    case "left":
      return "right";

    case "right":
      return "left";

    default:
      console.error("invalid yAxisAt ".concat(yAxisAt));
  }
}

function getTextLeft(left, label, props) {
  var yAxisAt = props.yAxisAt;
  var textPadding = label.textPadding,
      width = label.width;

  switch (yAxisAt) {
    case "left":
      return left + width - textPadding;

    case "right":
      return left + textPadding;

    default:
      console.error("invalid yAxisAt ".concat(yAxisAt));
  }
}

function getLineStart$1(label, layout, props) {
  var graph = layout.graph;
  var yAxisAt = props.yAxisAt,
      yTickLength = props.yTickLength;

  if (yAxisAt == "left") {
    return graph.left - yTickLength;
  }

  if (yAxisAt == "right") {
    return graph.left + graph.width + yTickLength;
  }

  return 0;
}

function getLineEnd$1(label, layout, props) {
  var graph = layout.graph;
  var yAxisAt = props.yAxisAt;

  if (yAxisAt == "left") {
    return graph.left + graph.width;
  }

  if (yAxisAt == "right") {
    return graph.left;
  }

  return 0;
}

function draw$3(ctx, layout, label, props) {
  label = setDefaults$1(label);
  PropTypes.checkPropTypes(propTypes$1, label, "prop", "y-label");
  var _label = label,
      y = _label.y,
      width = _label.width,
      height = _label.height,
      backgroundColor = _label.backgroundColor,
      font = _label.font,
      color = _label.color,
      textPadding = _label.textPadding,
      render = _label.render,
      drawLine = _label.drawLine,
      lineWidth = _label.lineWidth,
      lineColor = _label.lineColor;
  var graph = layout.graph;
  var yMin = props.yMin,
      yMax = props.yMax;

  if (y == undefined) {
    return;
  }

  var canvasY = getCanvasY(graph.height, graph.top, yMax, yMin, y);
  var top = canvasY - Math.round(height / 2);
  var left = getLeft(label, layout, props);
  ctx.fillStyle = backgroundColor; // label box

  ctx.fillRect(left, top, width, height); // text

  ctx.font = font;
  ctx.fillStyle = color;
  ctx.textAlign = getTextAlign(props);
  ctx.textBaseline = "middle";
  ctx.fillText(render(y), getTextLeft(left, label, props), top + textPadding);

  if (drawLine) {
    ctx.lineWidth = lineWidth;
    ctx.strokeStyle = lineColor;
    var lineStart = getLineStart$1(label, layout, props);
    var lineEnd = getLineEnd$1(label, layout, props);
    ctx.beginPath();
    ctx.moveTo(lineStart, top + height / 2);
    ctx.lineTo(lineEnd, top + height / 2);
    ctx.stroke();
  }
}

var yLabel = /*#__PURE__*/Object.freeze({
  draw: draw$3
});

var propTypes$2 = {
  left: PropTypes.number.isRequired,
  top: PropTypes.number.isRequired,
  color: PropTypes.string.isRequired,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
  font: PropTypes.string.isRequired
};
var defaultProps$2 = {
  color: "black",
  font: "12px Arial"
};

function setDefaults$2(props) {
  return _objectSpread2({}, defaultProps$2, {}, props);
}

function draw$4(ctx, props) {
  props = setDefaults$2(props);
  var _props = props,
      left = _props.left,
      top = _props.top,
      color = _props.color,
      font = _props.font,
      text = _props.text;
  PropTypes.checkPropTypes(propTypes$2, props, "prop", "text");
  ctx.textBaseline = "top";
  ctx.textAlign = "left";
  ctx.font = font;
  ctx.fillStyle = color;
  ctx.fillText("".concat(text), left, top);
}

var text = /*#__PURE__*/Object.freeze({
  draw: draw$4
});

var propTypes$3 = {
  canvasX: PropTypes.number,
  canvasY: PropTypes.number,
  xLineColor: PropTypes.string.isRequired,
  xLineWidth: PropTypes.number.isRequired,
  yLineColor: PropTypes.string.isRequired,
  yLineWidth: PropTypes.number.isRequired
};
var defaultProps$3 = {
  xLineColor: "",
  xLineWidth: 0.5,
  yLineColor: "",
  yLineWidth: 0.5
};

function setDefaults$3(props) {
  return _objectSpread2({}, defaultProps$3, {}, props);
}

function draw$5(ctx, layout, props) {
  props = setDefaults$3(props);
  PropTypes.checkPropTypes(propTypes$3, props, "prop", "crosshair");
  var _layout$graph = layout.graph,
      width = _layout$graph.width,
      height = _layout$graph.height,
      left = _layout$graph.left,
      top = _layout$graph.top;
  var _props = props,
      canvasX = _props.canvasX,
      canvasY = _props.canvasY,
      xLineColor = _props.xLineColor,
      xLineWidth = _props.xLineWidth,
      yLineColor = _props.yLineColor,
      yLineWidth = _props.yLineWidth;

  if (!isInside({
    top: top,
    left: left,
    width: width,
    height: height
  }, {
    x: canvasX,
    y: canvasY
  })) {
    return;
  } // x line


  ctx.strokeStyle = xLineColor;
  ctx.lineWidth = xLineWidth;
  ctx.beginPath();
  ctx.moveTo(canvasX, top);
  ctx.lineTo(canvasX, top + height);
  ctx.stroke(); // y line

  ctx.strokeStyle = yLineColor;
  ctx.lineWidth = yLineWidth;
  ctx.beginPath();
  ctx.moveTo(left, canvasY);
  ctx.lineTo(left + width, canvasY);
  ctx.stroke();
}

var crosshair = /*#__PURE__*/Object.freeze({
  draw: draw$5
});

var propTypes$4 = {
  data: PropTypes.arrayOf(PropTypes.shape({
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  })).isRequired,
  step: PropTypes.number.isRequired,
  lineColor: PropTypes.string.isRequired
};
var defaultProps$4 = {
  lineColor: "",
  step: 1,
  data: []
};

function setDefaults$4(props) {
  return _objectSpread2({}, defaultProps$4, {}, props);
}

function draw$6(ctx, layout, graph, props) {
  graph = setDefaults$4(graph);
  PropTypes.checkPropTypes(propTypes$4, graph, "prop", "line");
  var _layout$graph = layout.graph,
      top = _layout$graph.top,
      left = _layout$graph.left,
      width = _layout$graph.width,
      height = _layout$graph.height;
  var _graph = graph,
      data = _graph.data,
      step = _graph.step,
      lineColor = _graph.lineColor;
  var xMin = props.xMin,
      xMax = props.xMax,
      yMin = props.yMin,
      yMax = props.yMax;
  ctx.strokeStyle = lineColor;
  ctx.lineWidth = 1;

  if (step > 0) {
    ctx.beginPath();

    for (var i = 0; i < data.length; i += step) {
      var _data$i = data[i],
          x = _data$i.x,
          y = _data$i.y;

      if (x >= xMin && x <= xMax) {
        ctx.lineTo(getCanvasX(width, left, xMax, xMin, x), getCanvasY(height, top, yMax, yMin, y));
      }
    }

    ctx.stroke();
  }
}

var line = /*#__PURE__*/Object.freeze({
  draw: draw$6
});

var propTypes$5 = {
  data: PropTypes.arrayOf(PropTypes.shape({
    x: PropTypes.number,
    y: PropTypes.number
  })).isRequired,
  color: PropTypes.string.isRequired,
  radius: PropTypes.number.isRequired,
  ambientColor: PropTypes.string.isRequired,
  ambientRadius: PropTypes.number.isRequired
};
var defaultProps$5 = {
  color: "black",
  radius: 2,
  ambientColor: "rgba(80, 80, 80, 0.3)",
  ambientRadius: 0,
  data: []
};

function setDefaults$5(props) {
  return _objectSpread2({}, defaultProps$5, {}, props);
}

function drawPoint(ctx, layout, graph, point, props) {
  var _layout$graph = layout.graph,
      top = _layout$graph.top,
      left = _layout$graph.left,
      width = _layout$graph.width,
      height = _layout$graph.height;
  var color = graph.color,
      radius = graph.radius,
      ambientColor = graph.ambientColor,
      ambientRadius = graph.ambientRadius;
  var xMin = props.xMin,
      xMax = props.xMax,
      yMin = props.yMin,
      yMax = props.yMax;
  var x = point.x,
      y = point.y;

  if (x === undefined || y === undefined) {
    return;
  }

  var canvasX = getCanvasX(width, left, xMax, xMin, x);
  var canvasY = getCanvasY(height, top, yMax, yMin, y);

  if (ambientRadius > 0) {
    ctx.beginPath();
    ctx.arc(canvasX, canvasY, ambientRadius, 0, 2 * Math.PI, false);
    ctx.fillStyle = ambientColor;
    ctx.fill();
  }

  if (radius > 0) {
    ctx.beginPath();
    ctx.arc(canvasX, canvasY, radius, 0, 2 * Math.PI, false);
    ctx.fillStyle = color;
    ctx.fill();
  }
}
function draw$7(ctx, layout, graph, props) {
  graph = setDefaults$5(graph);
  PropTypes.checkPropTypes(propTypes$5, graph, "prop", "points");
  var _graph = graph,
      data = _graph.data;
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = data[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var point = _step.value;
      drawPoint(ctx, layout, graph, point, props);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

var points = /*#__PURE__*/Object.freeze({
  drawPoint: drawPoint,
  draw: draw$7
});

var propTypes$6 = {
  data: PropTypes.arrayOf(PropTypes.shape({
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  })).isRequired,
  step: PropTypes.number.isRequired,
  getBarColor: PropTypes.func.isRequired,
  barWidth: PropTypes.number.isRequired
};
var defaultProps$6 = {
  step: 1,
  getBarColor: function getBarColor() {
    return "";
  },
  barWidth: 1
};

function setDefaults$6(props) {
  return _objectSpread2({}, defaultProps$6, {}, props);
}

function draw$8(ctx, layout, graph, props) {
  graph = setDefaults$6(graph);
  PropTypes.checkPropTypes(propTypes$6, graph, "prop", "bars");
  var _layout$graph = layout.graph,
      top = _layout$graph.top,
      left = _layout$graph.left,
      width = _layout$graph.width,
      height = _layout$graph.height;
  var _graph = graph,
      data = _graph.data,
      step = _graph.step,
      getBarColor = _graph.getBarColor,
      barWidth = _graph.barWidth;
  var xMin = props.xMin,
      xMax = props.xMax,
      yMin = props.yMin,
      yMax = props.yMax;
  var canvasY0 = getCanvasY(height, top, yMax, yMin, yMin);

  if (step > 0) {
    for (var i = 0; i < data.length; i += step) {
      var d = data[i];
      var x = d.x,
          y = d.y;

      if (x >= xMin && x <= xMax) {
        var canvasX = getCanvasX(width, left, xMax, xMin, x);
        var canvasY = getCanvasY(height, top, yMax, yMin, y);
        var barHeight = canvasY0 - canvasY;
        ctx.fillStyle = getBarColor(d);
        ctx.fillRect(canvasX - barWidth / 2, canvasY, barWidth, Math.max(0, barHeight - 1));
      }
    }
  }
}

var bars = /*#__PURE__*/Object.freeze({
  draw: draw$8
});

var propTypes$7 = {
  data: PropTypes.arrayOf(PropTypes.shape({
    open: PropTypes.number.isRequired,
    close: PropTypes.number.isRequired,
    high: PropTypes.number.isRequired,
    low: PropTypes.number.isRequired,
    timestamp: PropTypes.number.isRequired
  })).isRequired,
  step: PropTypes.number.isRequired,
  getColor: PropTypes.func.isRequired,
  candlestickWidth: PropTypes.number.isRequired,
  lineWidth: PropTypes.number.isRequired
};
var defaultProps$7 = {
  data: [],
  step: 1,
  getColor: function getColor(data) {
    return "green";
  },
  candlestickWidth: 5,
  lineWidth: 1
};

function setDefaults$7(props) {
  return _objectSpread2({}, defaultProps$7, {}, props);
}

function draw$9(ctx, layout, graph, props) {
  graph = setDefaults$7(graph);
  PropTypes.checkPropTypes(propTypes$7, graph, "prop", "candlesticks");
  var _layout$graph = layout.graph,
      top = _layout$graph.top,
      left = _layout$graph.left,
      width = _layout$graph.width,
      height = _layout$graph.height;
  var _graph = graph,
      data = _graph.data,
      step = _graph.step,
      getColor = _graph.getColor,
      candlestickWidth = _graph.candlestickWidth,
      lineWidth = _graph.lineWidth;
  var xMin = props.xMin,
      xMax = props.xMax,
      yMin = props.yMin,
      yMax = props.yMax;

  if (step > 0) {
    for (var i = 0; i < data.length; i += step) {
      var _data$i = data[i],
          high = _data$i.high,
          low = _data$i.low,
          open = _data$i.open,
          close = _data$i.close,
          timestamp = _data$i.timestamp;

      if (timestamp >= xMin && timestamp <= xMax) {
        var canvasX = getCanvasX(width, left, xMax, xMin, timestamp);
        var bodyTop = getCanvasY(height, top, yMax, yMin, Math.max(open, close));
        var bodyBottom = getCanvasY(height, top, yMax, yMin, Math.min(open, close));
        var bodyHeight = Math.max(bodyBottom - bodyTop, 1);
        ctx.fillStyle = getColor(data[i]); // body

        ctx.fillRect(canvasX - candlestickWidth / 2, bodyTop, candlestickWidth, bodyHeight);
        ctx.strokeStyle = ctx.fillStyle;
        ctx.lineWidth = lineWidth; // top wick

        ctx.beginPath();
        ctx.moveTo(canvasX, bodyTop);
        ctx.lineTo(canvasX, getCanvasY(height, top, yMax, yMin, high)); // bottom wick

        ctx.moveTo(canvasX, bodyTop + bodyHeight);
        ctx.lineTo(canvasX, getCanvasY(height, top, yMax, yMin, low));
        ctx.stroke();
      }
    }
  }
}

var candlesticks = /*#__PURE__*/Object.freeze({
  draw: draw$9
});

var propTypes$8 = {
  graph: PropTypes.shape({
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired
  }).isRequired,
  xMin: PropTypes.number.isRequired,
  xMax: PropTypes.number.isRequired,
  yMin: PropTypes.number.isRequired,
  yMax: PropTypes.number.isRequired,
  data: PropTypes.arrayOf(PropTypes.shape({
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  })).isRequired,
  lineColor: PropTypes.string.isRequired,
  lineWidth: PropTypes.number.isRequired,
  step: PropTypes.number.isRequired
};
function draw$a(ctx, props) {
  var graph = props.graph,
      xMin = props.xMin,
      xMax = props.xMax,
      yMin = props.yMin,
      yMax = props.yMax,
      data = props.data,
      lineColor = props.lineColor,
      lineWidth = props.lineWidth,
      _props$step = props.step,
      step = _props$step === void 0 ? 1 : _props$step;
  PropTypes.checkPropTypes(propTypes$8, props, "prop", "graph");
  ctx.strokeStyle = lineColor;
  ctx.lineWidth = lineWidth;
  ctx.beginPath();

  for (var i = 0; i < data.length; i += step) {
    var _data$i = data[i],
        x = _data$i.x,
        y = _data$i.y;
    var canvasX = getCanvasX(graph.width, 0, xMax, xMin, x);
    var canvasY = getCanvasY(graph.height, 0, yMax, yMin, y);
    ctx.lineTo(canvasX, canvasY);
  }

  ctx.stroke();
}

var graph = /*#__PURE__*/Object.freeze({
  draw: draw$a
});

var propTypes$9 = {
  graph: PropTypes.shape({
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired
  }).isRequired,
  mask: PropTypes.shape({
    left: PropTypes.number.isRequired,
    width: PropTypes.number.isRequired
  }).isRequired,
  maskColor: PropTypes.string.isRequired
};
function draw$b(ctx, props) {
  var graph = props.graph,
      mask = props.mask,
      maskColor = props.maskColor;
  PropTypes.checkPropTypes(propTypes$9, props, "prop", "mask");
  ctx.fillStyle = maskColor;
  ctx.fillRect(mask.left, 0, mask.width, graph.height);
}

var mask = /*#__PURE__*/Object.freeze({
  draw: draw$b
});

var TICK_TEXT_TOP_PADDING = 10;
var propTypes$a = {
  graph: PropTypes.shape({
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired
  }).isRequired,
  xAxisColor: PropTypes.string.isRequired,
  xMin: PropTypes.number.isRequired,
  xMax: PropTypes.number.isRequired,
  tickHeight: PropTypes.number.isRequired,
  ticks: PropTypes.arrayOf(PropTypes.number).isRequired,
  renderTick: PropTypes.func.isRequired,
  font: PropTypes.string.isRequired,
  textColor: PropTypes.string.isRequired
};
function draw$c(ctx, props) {
  var graph = props.graph,
      xAxisColor = props.xAxisColor,
      xMin = props.xMin,
      xMax = props.xMax,
      tickHeight = props.tickHeight,
      ticks = props.ticks,
      renderTick = props.renderTick,
      font = props.font,
      textColor = props.textColor;
  PropTypes.checkPropTypes(propTypes$a, props, "prop", "x-axis"); // style x axis

  ctx.lineWidth = 1;
  ctx.strokeStyle = xAxisColor;
  ctx.beginPath();
  ctx.moveTo(0, graph.height);
  ctx.lineTo(graph.width, graph.height);
  ctx.stroke(); // style ticks

  ctx.font = font;
  ctx.fillStyle = textColor;
  ctx.textAlign = "center";
  ctx.textBaseline = "middle";
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = ticks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var tick = _step.value;
      var canvasX = getCanvasX(graph.width, 0, xMax, xMin, tick);
      ctx.beginPath();
      ctx.moveTo(canvasX, graph.height);
      ctx.lineTo(canvasX, graph.height + tickHeight);
      ctx.stroke();
      ctx.fillText(renderTick(tick), canvasX, graph.height + tickHeight + TICK_TEXT_TOP_PADDING);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

var xAxis$1 = /*#__PURE__*/Object.freeze({
  draw: draw$c
});

var history = {
  graph: graph,
  mask: mask,
  xAxis: xAxis$1
};

var history$1 = /*#__PURE__*/Object.freeze({
  'default': history
});

var canvas = {
  xAxis: xAxis,
  yAxis: yAxis,
  xLabel: xLabel,
  yLabel: yLabel,
  text: text,
  crosshair: crosshair,
  line: line,
  points: points,
  bars: bars,
  candlesticks: candlesticks,
  math: math,
  history: history$1
};

function getLayout(props) {
  var padding = props.padding,
      width = props.width,
      height = props.height,
      xAxisAt = props.xAxisAt,
      _props$xAxisHeight = props.xAxisHeight,
      xAxisHeight = _props$xAxisHeight === void 0 ? 0 : _props$xAxisHeight,
      yAxisAt = props.yAxisAt,
      _props$yAxisWidth = props.yAxisWidth,
      yAxisWidth = _props$yAxisWidth === void 0 ? 0 : _props$yAxisWidth;
  var xAxis = {
    top: xAxisAt == "top" ? padding : height - padding - xAxisHeight,
    left: yAxisAt == "left" ? padding + yAxisWidth : padding,
    width: width - 2 * padding - yAxisWidth,
    height: xAxisHeight
  };
  var yAxis = {
    top: xAxisAt == "top" ? padding + xAxisHeight : padding,
    left: yAxisAt == "left" ? padding : width - padding - yAxisWidth,
    width: yAxisWidth,
    height: height - 2 * padding - xAxisHeight
  };
  var graph = {
    top: xAxisAt == "top" ? padding + xAxisHeight : padding,
    left: yAxisAt == "left" ? padding + yAxisWidth : padding,
    width: width - 2 * padding - yAxisWidth,
    height: height - 2 * padding - xAxisHeight
  };
  return {
    graph: graph,
    xAxis: xAxis,
    yAxis: yAxis
  };
}
var layout = {
  getLayout: getLayout
};

var GRAPHS = {
  line: line,
  points: points,
  bars: bars,
  candlesticks: candlesticks
};

var Graph =
/*#__PURE__*/
function (_Component) {
  _inherits(Graph, _Component);

  function Graph(props) {
    var _this;

    _classCallCheck(this, Graph);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Graph).call(this, props));

    _defineProperty(_assertThisInitialized(_this), "draw", function () {
      var _this$props = _this.props,
          xMin = _this$props.xMin,
          xMax = _this$props.xMax,
          yMin = _this$props.yMin,
          yMax = _this$props.yMax,
          xAxisAt = _this$props.xAxisAt,
          yAxisAt = _this$props.yAxisAt;

      _this.ctx.axes.clearRect(0, 0, _this.props.width, _this.props.height);

      if (xAxisAt) {
        draw(_this.ctx.axes, _this.layout, _this.props);
      }

      if (yAxisAt) {
        draw$1(_this.ctx.axes, _this.layout, _this.props);
      }

      _this.ctx.graphs.clearRect(0, 0, _this.props.width, _this.props.height);

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = _this.props.graphs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var graph = _step.value;
          GRAPHS[graph.type].draw(_this.ctx.graphs, _this.layout, graph, _this.props);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      _this.ctx.ui.clearRect(0, 0, _this.props.width, _this.props.height);

      if (_this.props.crosshair) {
        draw$5(_this.ctx.ui, _this.layout, _this.props.crosshair);
      }

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = _this.props.frames[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var frame = _step2.value;
          draw$4(_this.ctx.ui, frame);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
            _iterator2["return"]();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = _this.props.xLabels[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var label = _step3.value;
          draw$2(_this.ctx.ui, _this.layout, label, _this.props);
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
            _iterator3["return"]();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = _this.props.yLabels[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var _label = _step4.value;
          draw$3(_this.ctx.ui, _this.layout, _label, _this.props);
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4["return"] != null) {
            _iterator4["return"]();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }
    });

    _defineProperty(_assertThisInitialized(_this), "animate", function () {
      _this.animation = window.requestAnimationFrame(_this.animate);

      _this.draw();
    });

    _defineProperty(_assertThisInitialized(_this), "getMouse", function (e) {
      var rect = _this.ctx.ui.canvas.getBoundingClientRect();

      return {
        x: e.clientX - rect.left,
        y: e.clientY - rect.top
      };
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseMove", function (e) {
      _this.props.onMouseMove(e, _this.getMouse(e), _this.layout.graph);
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseDown", function (e) {
      _this.props.onMouseDown(e, _this.getMouse(e), _this.layout.graph);
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseUp", function (e) {
      _this.props.onMouseUp(e, _this.getMouse(e));
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseOut", function (e) {
      _this.props.onMouseOut(e);
    });

    _defineProperty(_assertThisInitialized(_this), "onWheel", function (e) {
      _this.props.onWheel(e, _this.getMouse(e), _this.layout.graph);
    });

    _this.axes = React.createRef();
    _this.graphs = React.createRef();
    _this.ui = React.createRef(); // ref to animation frame

    _this.animation = undefined; // cache layout

    _this.layout = getLayout(props);
    return _this;
  }

  _createClass(Graph, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.ctx = {
        axes: this.axes.current.getContext("2d"),
        graphs: this.graphs.current.getContext("2d"),
        ui: this.ui.current.getContext("2d")
      };

      if (this.props.animate) {
        this.animate();
      } else {
        this.draw();
      }
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      if (!this.props.animate && this.props.shouldRedrawGraph()) {
        this.draw();
      }

      this.layout = getLayout(this.props);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      window.cancelAnimationFrame(this.animation);
    }
  }, {
    key: "render",
    value: function render() {
      return React.createElement("div", {
        style: _objectSpread2({}, styles.container, {
          width: this.props.width,
          height: this.props.height,
          backgroundColor: this.props.backgroundColor
        })
      }, React.createElement("canvas", {
        ref: this.axes,
        style: styles.canvas,
        width: this.props.width,
        height: this.props.height
      }), React.createElement("canvas", {
        ref: this.graphs,
        style: styles.canvas,
        width: this.props.width,
        height: this.props.height
      }), React.createElement("canvas", {
        ref: this.ui,
        style: styles.canvas,
        width: this.props.width,
        height: this.props.height,
        onMouseMove: this.onMouseMove,
        onMouseDown: this.onMouseDown,
        onMouseUp: this.onMouseUp,
        onMouseOut: this.onMouseOut,
        onWheel: this.onWheel
      }));
    }
  }]);

  return Graph;
}(Component);

Graph.defaultProps = {
  width: 500,
  height: 300,
  padding: 10,
  animate: true,
  shouldRedrawGraph: function shouldRedrawGraph() {
    return true;
  },
  backgroundColor: "",
  // x axis
  xAxisHeight: 30,
  xAxisLineColor: "black",
  xTicks: [],
  xTickInterval: 0,
  xTickLength: 10,
  renderXTick: function renderXTick(x) {
    return x;
  },
  xAxisFont: "",
  xAxisTextColor: "black",
  showXLine: true,
  xLineColor: "lightgrey",
  // y axis
  yAxisWidth: 50,
  yAxisLineColor: "black",
  yTicks: [],
  yTickInterval: 0,
  yTickLength: 10,
  renderYTick: function renderYTick(y) {
    return y;
  },
  yAxisFont: "",
  yAxisTextColor: "black",
  showYLine: true,
  yLineColor: "lightgrey",
  xMin: 0,
  xMax: 0,
  yMin: 0,
  yMax: 0,
  graphs: [],
  frames: [],
  xLabels: [],
  yLabels: [],
  onMouseMove: function onMouseMove() {},
  onMouseDown: function onMouseDown() {},
  onMouseUp: function onMouseUp() {},
  onMouseOut: function onMouseOut() {},
  onWheel: function onWheel() {}
};
Graph.propTypes = {
  width: PropTypes.number.isRequired,
  height: PropTypes.number.isRequired,
  padding: PropTypes.number.isRequired,
  backgroundColor: PropTypes.string.isRequired,
  animate: PropTypes.bool.isRequired,
  shouldRedrawGraph: PropTypes.func.isRequired,
  xMin: PropTypes.number.isRequired,
  xMax: PropTypes.number.isRequired,
  yMin: PropTypes.number.isRequired,
  yMax: PropTypes.number.isRequired,
  // x axis
  xAxisAt: PropTypes.oneOf(["top", "bottom"]),
  xAxisHeight: PropTypes.number.isRequired,
  xAxisLineColor: PropTypes.string.isRequired,
  xTicks: PropTypes.arrayOf(PropTypes.number.isRequired).isRequired,
  xTickInterval: PropTypes.number.isRequired,
  xTickLength: PropTypes.number.isRequired,
  renderXTick: PropTypes.func.isRequired,
  xAxisFont: PropTypes.string.isRequired,
  xAxisTextColor: PropTypes.string.isRequired,
  showXLine: PropTypes.bool.isRequired,
  xLineColor: PropTypes.string.isRequired,
  // y axis
  yAxisAt: PropTypes.oneOf(["left", "right"]),
  yAxisWidth: PropTypes.number.isRequired,
  yAxisLineColor: PropTypes.string.isRequired,
  yTicks: PropTypes.arrayOf(PropTypes.number.isRequired).isRequired,
  yTickInterval: PropTypes.number.isRequired,
  yTickLength: PropTypes.number.isRequired,
  renderYTick: PropTypes.func.isRequired,
  yAxisFont: PropTypes.string.isRequired,
  yAxisTextColor: PropTypes.string.isRequired,
  showYLine: PropTypes.bool.isRequired,
  yLineColor: PropTypes.string.isRequired,
  // graphs
  graphs: PropTypes.arrayOf(PropTypes.shape({
    type: PropTypes.oneOf(["line", "points", "bars", "candlesticks"])
  })).isRequired,
  frames: PropTypes.array.isRequired,
  xLabels: PropTypes.array.isRequired,
  yLabels: PropTypes.array.isRequired,
  crosshair: PropTypes.object,
  onMouseMove: PropTypes.func.isRequired,
  onMouseDown: PropTypes.func.isRequired,
  onMouseUp: PropTypes.func.isRequired,
  onMouseOut: PropTypes.func.isRequired,
  onWheel: PropTypes.func.isRequired
};
var styles = {
  container: {
    position: "relative",
    cursor: "crosshair"
  },
  canvas: {
    position: "absolute",
    left: 0,
    top: 0
  }
};

var History =
/*#__PURE__*/
function (_Component) {
  _inherits(History, _Component);

  function History(props) {
    var _this;

    _classCallCheck(this, History);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(History).call(this, props));

    _defineProperty(_assertThisInitialized(_this), "getGraph", function () {
      return {
        top: 0,
        left: 0,
        width: _this.props.width,
        height: _this.props.height - _this.props.xAxisHeight
      };
    });

    _defineProperty(_assertThisInitialized(_this), "drawGraph", function () {
      _this.ctx.graph.clearRect(0, 0, _this.props.width, _this.props.height);

      var _this$props = _this.props,
          data = _this$props.data,
          xMin = _this$props.xMin,
          xMax = _this$props.xMax,
          yMin = _this$props.yMin,
          yMax = _this$props.yMax;

      var graph = _this.getGraph();

      history.xAxis.draw(_this.ctx.graph, _objectSpread2({}, _this.props, {
        graph: graph,
        xMin: xMin,
        xMax: xMax
      }));
      history.graph.draw(_this.ctx.graph, _objectSpread2({}, _this.props, {
        graph: graph,
        xMin: xMin,
        xMax: xMax,
        yMin: yMin,
        yMax: yMax
      }));
    });

    _defineProperty(_assertThisInitialized(_this), "animate", function () {
      _this.animation = window.requestAnimationFrame(_this.animate);

      _this.ctx.ui.clearRect(0, 0, _this.props.width, _this.props.height);

      var graph = _this.getGraph();

      history.mask.draw(_this.ctx.ui, _objectSpread2({}, _this.props, {
        graph: graph
      }));
    });

    _defineProperty(_assertThisInitialized(_this), "getMouse", function (e) {
      var rect = _this.ctx.ui.canvas.getBoundingClientRect();

      return {
        x: e.clientX - rect.left,
        y: e.clientY - rect.top
      };
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseMove", function (e) {
      _this.props.onMouseMove(e, _this.getMouse(e));
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseDown", function (e) {
      _this.props.onMouseDown(e, _this.getMouse(e));
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseUp", function (e) {
      _this.props.onMouseUp(e, _this.getMouse(e));
    });

    _defineProperty(_assertThisInitialized(_this), "onMouseOut", function (e) {
      _this.props.onMouseOut(e);
    });

    _this.ui = React.createRef();
    _this.graph = React.createRef(); // ref to animation frame

    _this.animation = undefined;
    return _this;
  }

  _createClass(History, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.ctx = {
        ui: this.ui.current.getContext("2d"),
        graph: this.graph.current.getContext("2d")
      };
      this.drawGraph();
      this.animate();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      window.cancelAnimationFrame(this.animation);
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      if (this.props.shouldRedrawGraph()) {
        this.drawGraph();
      }
    }
  }, {
    key: "getCursor",
    value: function getCursor() {
      var _this$props2 = this.props,
          maskEdgeDelta = _this$props2.maskEdgeDelta,
          mouse = _this$props2.mouse,
          mask = _this$props2.mask;
      var graph = this.getGraph();

      if (!isInside(graph, mouse)) {
        return "auto";
      }

      if (Math.abs(mask.left - mouse.x) <= maskEdgeDelta || Math.abs(mask.left + mask.width - mouse.x) <= maskEdgeDelta) {
        return "ew-resize";
      }

      if (mouse.x > mask.left && mouse.x < mask.left + mask.width) {
        return "grab";
      }

      return "auto";
    }
  }, {
    key: "render",
    value: function render() {
      var cursor = this.getCursor();
      return React.createElement("div", {
        style: _objectSpread2({}, styles$1.container, {
          cursor: cursor,
          backgroundColor: this.props.backgroundColor,
          width: this.props.width,
          height: this.props.height
        })
      }, React.createElement("canvas", {
        ref: this.graph,
        style: styles$1.canvas,
        width: this.props.width,
        height: this.props.height
      }), React.createElement("canvas", {
        ref: this.ui,
        style: styles$1.canvas,
        width: this.props.width,
        height: this.props.height,
        onMouseMove: this.onMouseMove,
        onMouseDown: this.onMouseDown,
        onMouseUp: this.onMouseUp,
        onMouseOut: this.onMouseOut
      }));
    }
  }]);

  return History;
}(Component);

var styles$1 = {
  container: {
    position: "relative",
    cursor: "auto"
  },
  canvas: {
    position: "absolute",
    left: 0,
    top: 0
  }
};
History.defaultProps = {
  shouldRedrawGraph: function shouldRedrawGraph() {
    return true;
  },
  width: 800,
  height: 200,
  backgroundColor: "",
  xAxisHeight: 30,
  xAxisColor: "",
  xMin: 0,
  xMax: 0,
  yMin: 0,
  yMax: 0,
  tickHeight: 5,
  ticks: [],
  renderTick: function renderTick(x) {
    return x;
  },
  font: "12px Arial",
  textColor: "",
  // graph
  data: [],
  lineColor: "",
  lineWidth: 1,
  step: 1,
  // mask
  maskColor: "rgba(100, 100, 100, 0.5)",
  mask: {
    left: 0,
    width: 0
  },
  maskEdgeDelta: 10,
  mouse: {
    x: undefined,
    y: undefined
  },
  onMouseMove: function onMouseMove() {},
  onMouseDown: function onMouseDown() {},
  onMouseUp: function onMouseUp() {},
  onMouseOut: function onMouseOut() {}
};
History.propTypes = {
  shouldRedrawGraph: PropTypes.func.isRequired,
  backgroundColor: PropTypes.string.isRequired,
  width: PropTypes.number.isRequired,
  height: PropTypes.number.isRequired,
  xAxisHeight: PropTypes.number.isRequired,
  xAxisColor: PropTypes.string.isRequired,
  tickHeight: PropTypes.number.isRequired,
  ticks: PropTypes.arrayOf(PropTypes.number).isRequired,
  renderTick: PropTypes.func.isRequired,
  font: PropTypes.string.isRequired,
  textColor: PropTypes.string.isRequired,
  xMin: PropTypes.number.isRequired,
  xMax: PropTypes.number.isRequired,
  yMin: PropTypes.number.isRequired,
  yMax: PropTypes.number.isRequired,
  // graph
  data: PropTypes.arrayOf(PropTypes.shape({
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  })).isRequired,
  lineColor: PropTypes.string.isRequired,
  lineWidth: PropTypes.number.isRequired,
  step: PropTypes.number.isRequired,
  // mask
  maskColor: PropTypes.string.isRequired,
  mask: PropTypes.shape({
    left: PropTypes.number.isRequired,
    width: PropTypes.number.isRequired
  }).isRequired,
  maskEdgeDelta: PropTypes.number.isRequired,
  mouse: PropTypes.shape({
    x: PropTypes.number,
    y: PropTypes.number
  }).isRequired,
  onMouseMove: PropTypes.func,
  onMouseDown: PropTypes.func,
  onMouseUp: PropTypes.func,
  onMouseOut: PropTypes.func
};

function draggable(Component) {
  function Draggable(props) {
    var _useState = useState({
      dragging: false
    }),
        _useState2 = _slicedToArray(_useState, 2),
        state = _useState2[0],
        setState = _useState2[1];

    function getXRange(mouse, graph) {
      if (!canvas.math.isInside(graph, mouse) || !state.dragging) {
        return {
          xMin: props.xMin,
          xMax: props.xMax
        };
      }

      var diff = mouse.x - props.mouse.x;
      var xMin = canvas.math.getX(graph.width, graph.left, props.xMax, props.xMin, graph.left - diff);
      var xMax = canvas.math.getX(graph.width, graph.left, props.xMax, props.xMin, graph.width + graph.left - diff);
      return {
        xMin: xMin,
        xMax: xMax
      };
    }

    function onMouseMove(e, mouse, graph) {
      var _getXRange = getXRange(mouse, graph),
          xMin = _getXRange.xMin,
          xMax = _getXRange.xMax;

      props.onMouseMove(e, mouse, graph, {
        xMin: xMin,
        xMax: xMax
      });
    }

    function onMouseDown(e, mouse, graph) {
      if (canvas.math.isInside(graph, mouse)) {
        setState({
          dragging: true
        });
      }

      props.onMouseDown(e, mouse);
    }

    function onMouseUp(e, mouse) {
      setState({
        dragging: false
      });
      props.onMouseUp(e, mouse);
    }

    function onMouseOut() {
      setState({
        dragging: false
      });
      props.onMouseOut();
    }

    return React.createElement(Component, _extends({}, props, {
      onMouseMove: onMouseMove,
      onMouseOut: onMouseOut,
      onMouseDown: onMouseDown,
      onMouseUp: onMouseUp
    }));
  }

  Draggable.propTypes = {
    xMin: PropTypes.number.isRequired,
    xMax: PropTypes.number.isRequired,
    mouse: PropTypes.shape({
      x: PropTypes.number,
      y: PropTypes.number
    }).isRequired,
    onMouseMove: PropTypes.func.isRequired,
    onMouseDown: PropTypes.func.isRequired,
    onMouseUp: PropTypes.func.isRequired,
    onMouseOut: PropTypes.func.isRequired
  };
  Draggable.defaultProps = {
    onMouseMove: function onMouseMove() {},
    onMouseDown: function onMouseDown() {},
    onMouseUp: function onMouseUp() {},
    onMouseOut: function onMouseOut() {}
  };
  return Draggable;
}

var DEFAULT_ZOOM_RATE = 0.1;
function zoomable(Component) {
  function Zoomable(props) {
    var xMin = props.xMin,
        xMax = props.xMax,
        mouse = props.mouse,
        zoomRate = props.zoomRate;

    function getXRange(e, graph) {
      if (!canvas.math.isInside(graph, mouse)) {
        return {
          xMin: xMin,
          xMax: xMax
        };
      }

      var deltaY = e.deltaY;
      var x = canvas.math.getX(graph.width, graph.left, xMax, xMin, mouse.x);

      if (deltaY > 0) {
        // zoom out
        return {
          xMin: x - (x - xMin) * (1 + zoomRate),
          xMax: x + (xMax - x) * (1 + zoomRate)
        };
      } else {
        // zoom in
        return {
          xMin: x - (x - xMin) * (1 - zoomRate),
          xMax: x + (xMax - x) * (1 - zoomRate)
        };
      }
    }

    function onWheel(e, mouse, graph) {
      var _getXRange = getXRange(e, graph),
          xMin = _getXRange.xMin,
          xMax = _getXRange.xMax;

      props.onWheel(e, mouse, graph, {
        xMin: xMin,
        xMax: xMax
      });
    }

    return React.createElement(Component, _extends({}, props, {
      onWheel: onWheel
    }));
  }

  Zoomable.propTypes = {
    xMin: PropTypes.number.isRequired,
    xMax: PropTypes.number.isRequired,
    mouse: PropTypes.shape({
      x: PropTypes.number,
      y: PropTypes.number
    }).isRequired,
    onWheel: PropTypes.func.isRequired,
    zoomRate: PropTypes.number.isRequired
  };
  Zoomable.defaultProps = {
    zoomRate: DEFAULT_ZOOM_RATE,
    onWheel: function onWheel() {}
  };
  return Zoomable;
}

var ReactCanvasTimeSeries = {
  canvas: canvas,
  History: History,
  Graph: Graph,
  layout: layout,
  draggable: draggable,
  zoomable: zoomable
};

export default ReactCanvasTimeSeries;
export { Graph, History, canvas, draggable, layout, zoomable };
//# sourceMappingURL=index.es.js.map
